@@ -3,8 +3,7 @@ package AnyEvent::RabbitMQ::LocalQueue;
 use strict;
 use warnings;
 
-our $VERSION = '1.21_01'; # TRIAL VERSION
-$VERSION = eval $VERSION;
+our $VERSION = '1.03';
 
 sub new {
     my $class = shift;
@@ -46,16 +45,5 @@ sub _drain_queue {
     return $self;
 }
 
-sub _flush {
-    my ($self, $frame) = @_;
-
-    $self->_drain_queue;
-
-    while (my $cb = shift @{$self->{_drain_code_queue}}) {
-        local $@; # Flush frames immediately, throwing away errors for on-close
-        eval { $cb->($frame) };
-    }
-}
-
 1;
 
